@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: views.py 12678 2021-01-07 18:15:03Z Lavender $
+# $Id: views.py 12785 2021-02-25 16:33:33Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2021-01-08 02:15:03 +0800 (Fri, 08 Jan 2021) $
-# $Revision: 12678 $ 
+# $Date: 2021-02-26 00:33:33 +0800 (Fri, 26 Feb 2021) $
+# $Revision: 12785 $ 
 
 import json
 import logging
@@ -1919,6 +1919,191 @@ class ATMFillView(generic.View):
             }
         )
         
+import os
+import datetime
+import hashlib
+import textwrap
+
+from io import BytesIO
+
+from django.http import FileResponse
+
+from PIL import Image
+from PIL import ImageFont
+from PIL import ImageDraw 
+
+MAX_LENGTH = 60
+        
+class InvoiceView(generic.View):
+
+    def get(self, request, *args, **kwargs):
+        orderNumber = kwargs.get('orderNumber')
+        order = get_object_or_404(Order, number=orderNumber)
+        
+        if order.user:
+            if not order.user == request.user:
+                raise Http404()
+        
+        # data
+        if hasattr(settings, "IUNO_SHOP_INVOICE_LOGO"):
+            logo = settings.IUNO_SHOP_INVOICE_LOGO 
+        else:
+            logo = "Logo.png" 
+            
+        items = []
+        for line in order.basket.all_lines():
+            product = line.product
+            productName = product.get_title()
+            
+            items.append((
+                line.quantity,
+                productName,
+                float(line.price_excl_tax) * line.quantity,
+            ))
+            
+        if float(order.shipping_excl_tax) > 0:
+            items.append((
+                "",
+                str(_("Shipping total")),
+                float(order.shipping_excl_tax),
+            ))
+        
+        if order.user:
+            email = order.user.email
+        else:
+            email = order.guest_email
+        
+        if hasattr(settings, "IUNO_SHOP_INVOICE_FROM"):
+            fromContent = settings.IUNO_SHOP_INVOICE_FROM
+        else:
+            fromContent = settings.OSCAR_SHOP_NAME
+        orderNumber = order.number
+        date = order.date_placed.strftime("%d %b %Y")
+        dollarName = order.currency
+        total = order.total_excl_tax
+        
+        if Source.objects.filter(order=order).exists():
+            source = Source.objects.get(order=order)
+            pay = source.source_type.name
+        else:
+            pay = str(_("Cash"))
+
+        # https://stackoverflow.com/questions/16008670/how-to-hash-a-string-into-8-digits
+        invoice = "IVBF%s" % (int(hashlib.sha1(orderNumber.encode()).hexdigest(), 16) % (10 ** 8))
+
+        try:
+            font = ImageFont.truetype("msjh.ttf", 16) # win7 and centos
+        except Exception as e:
+            try:
+                font = ImageFont.truetype("msjh.ttc", 16) # win10
+            except Exception as e:
+                raise RuntimeError("msjh.ttf or msjh.ttc should be installed.")
+
+
+        # background
+        # 分辨率是150像素/英寸時，A4紙的尺寸的圖像的像素是1240×1754。
+        background = Image.new('RGB', (1240, 1754), color=(255, 255, 255))
+        draw = ImageDraw.Draw(background)
+
+        # logo
+        if os.path.exists(logo):
+            logo = Image.open(logo)
+            background.paste(logo, (200, 100))
+
+        # background content
+        color = (0, 0, 0) # 黑
+        draw.text((200, 200), str(_("To:")), color, font=font)
+        draw.text((600, 200), str(_("From:")), color, font=font)
+        draw.text((800, 100), str(_("Date:")), color, font=font)
+        draw.text((800, 130), str(_("Invoice No:")), color, font=font)
+        draw.text((800, 160), str(_("Order No:")), color, font=font)
+
+        # order item background
+        shape = [
+            (200, 400), 
+            (1100, 600 + (30 * sum([len(textwrap.wrap(item[1], width=MAX_LENGTH)) for item in items])))
+        ]
+        color = (230, 230, 230) # 白灰
+        draw.rectangle(shape, fill=color) 
+
+        # To
+        color = (87, 155, 255) # 水藍
+        draw.text((200, 230), email, color, font=font)
+        width, height = draw.textsize(email, font=font)
+        draw.line((200, 230 + height - 2, 200 + width, 230 + height - 2), fill=color)
+
+        # From
+        color = (135, 135, 135) # 灰
+        draw.text((600, 230), fromContent, color, font=font)
+
+        # date
+        draw.text((950, 100), date, color, font=font)
+
+        # invoice
+        draw.text((950, 130), invoice, color, font=font)
+
+        # Order No
+        draw.text((950, 160), orderNumber, color, font=font)
+
+        # Qty
+        color = (0, 0, 0) # 黑
+        draw.text((250, 450), str(_("Qty")), color, font=font)
+
+        # Description
+        draw.text((350, 450), str(_("Description")), color, font=font)
+
+        # Amount
+        draw.text((950, 450), str(_("Amount")), color, font=font)
+
+        # items
+        color = (135, 135, 135) # 灰
+        totalHeight = 0
+        for item in items:
+            description = textwrap.wrap(item[1], width=MAX_LENGTH)
+            
+            # Qty
+            draw.text((250, 500 + totalHeight), str(item[0]), color, font=font)
+
+            # Description
+            draw.text((350, 500 + totalHeight), "\n".join(description), color, font=font)
+
+            # Amount
+            draw.text((950, 500 + totalHeight), "$%.2f" % item[2], color, font=font)
+            
+            totalHeight += 30 * len(description)
+            
+
+        # Invoice Total: USD $100.00
+        color = (0, 0, 0) # 黑
+        draw.text(
+            (810, 520 + totalHeight), 
+            str(_("Invoice Total: %s $%.2f")) % (dollarName, total), 
+            color, font=font
+        )
+
+        # Paid via PayPal
+        color = (135, 135, 135) # 灰
+        draw.text(
+            (810, 550 + totalHeight), 
+            str(_("Paid via %s")) % pay, 
+            color, font=font
+        )
+
+        # All amounts shown on this invoice are in US dollars.
+        draw.text(
+            (420, 620 + totalHeight), 
+            str(_("All amounts shown on this invoice are in %s.")) % dollarName, 
+            color, font=font
+        )
+
+        # response
+        imagefile = BytesIO() 
+        background.save(imagefile, "PDF", resolution=100.0)
+        response = HttpResponse(imagefile.getvalue(), content_type='application/pdf')
+        response['Content-Disposition'] = ("filename=%s" % ("Invoice%s.pdf" % orderNumber,))
+        
+        return response
+        
 
 
 
