@@ -281,7 +281,9 @@ class SeleniumMixin(object):
     HEADLESS = True
     PORT = "9000"
     live_server_url = "http://127.0.0.1:%s" % PORT
-    
+    CMS_LOADDATA = False
+    LOADDATA_WITH_PARAMETER = ''
+
     @staticmethod
     def _setUpClass(cls):
         try:
@@ -326,7 +328,13 @@ class SeleniumMixin(object):
             os.remove(TEST_DB)
         
         os.system('"%s" manage.py migrate --settings %s' % (envPython, cls.TEST_SETTINGS))
-        os.system('"%s" manage.py loaddata %s --settings %s' % (envPython, cls.INITIAL_DATA, cls.TEST_SETTINGS))
+
+
+        if cls.CMS_LOADDATA:
+            os.system('"%s" manage.py cms_loaddata %s %s --settings %s' % (envPython, cls.INITIAL_DATA, cls.LOADDATA_WITH_PARAMETER, cls.TEST_SETTINGS))
+        else:
+            os.system('"%s" manage.py loaddata %s %s --settings %s' % (envPython, cls.INITIAL_DATA, cls.LOADDATA_WITH_PARAMETER, cls.TEST_SETTINGS))
+
        
         if cls.TEST_SETTINGS:
             p = subprocess.Popen(
